        module h_rbchemlocal
         use h_rbchemmech, only: c_kvec, c_cdim
        implicit none
        public
c=============================================================================
c
c $Id: rbchemlocal.h,v 1.4 2002/05/02 15:50:22 akinorii Exp $
c   
c CODE DEVELOPER
c   Sandy Sillman, U of M. (IMPACT version, 2007)
c 
c FILE  
c   rbchemlocal.h
c
c DESCRIPTION:  variable declarations and common blocks
c   for locally shared variables among the rb (quad) chem solver subroutines.
c   (Does not include allocated arrays)
c
c This file is included in the main chemistry solver routine
c   (quadchem.F in IMPACT)
c  but not in the entry-to-chemistry routine.
c
c COMMON BLOCKS have been removed for all variables that vary among grids
c  in order to fit with IMPACT multiple processor message passing
c  (on advice from Akinori Ito and Dan Bergman).
c 
c  The common variables will be passed as arguments.

c ==========================================================================
c 
c chemlocal.EXT    April, 2007
c   INCLUDE and COMMON file 
c     for RADICAL BALANCE-BACK EULER solver for chemistry (quadchem)
c      (chemmain.f cheminit.f chemrates chemsolve.f, linslv.f, jval2.f)
c   IMPACT VERSION. 
c
c  Note:  This file includes 'IMPLICIT NONE'. - cut.
c  Note:  This file uses KVEC.  
c    Its include statement must come AFTER chemmech.EXT
c 
c ichemlocal.h:  IMPACT version.  Local chem. variables
c   with large arrays moved to ichemlocarr.h  (to be allocated).  
c
c  This file includes declarations and common blocks 
c   for SOLVER VARIABLES 
c   which are shared among the solver subroutines in common blocks
c   but are not saved or passed to the main program.  

c  It also contains declarations for LOCAL VARIABLES
c   which are used in solver subroutines but not retained 
c   or passed to other subroutines  
c
c  Other INCLUDE files: 
c   chemvars.EXT =  input-output variables 
c    that are passed to and from the solver each time it is called.
c   chemmech.EXT =  variables for the chem. mechanism
c     that are set once and do not change as the mechanism is called.
c
c Program written by Sanford Sillman
c History:
c  12/06. Program written by Sandy Sillman based on commq7.EXT
c  7/07  IMPACT version (.h), arrays separated into ichemlocarr.h
c -------------------------------------------------------------
c IMPLICIT - added to each  subroutine
c      implicit none
c
c LOCAL SPECIES CONCENTRATION ARRAYS
c     (see also input-output variables in chemvars.h, chemarrays.h)
c   xc, xcfinr, xclastq
c  Moved to ichemlocarr.h:  
c  Note:  Allocate
c
c CHEMICAL REACTION RATES
c   ratek, rateh, rateq, rateqq, rhdif, egasaq. 
c   Note: Moved to chemlocarrs.  Allocate in chemmain1. or chemsolve)
c
c     (see also rate parameters in chemmech.EXT
c       and optional output in chemvars.EXT)

c
c  CHEMISTRY PARAMETERS (not in common; parameter values set here)
c
c  avogadrl   Avogadro's number * liters/cm3 =6.020e-20
c  atmos      Atmospheric surface pressure = 2.247E+19 mol/cm3
c               (used to convert Henry's law coeff moles/L-atm)
c  pii        Pi
c  rtcon      Conversion parameter for Henry's law rate calculations
c               R=8.314E-02. L-atm/mol-K   (8.206 in old program)
c  RUMOLEC    RU parameter for molecular speed (Barth, personal comm)
c               (8.314e7 g-cm2/s2-mol-K)   (note: 8.314e0 kg-m2/s2-mol-K)
c  DROPDIF    Droplet diffusion coefficient - 2e-5 cm2/sec, Lelieveld 1991
c  DIFGAS     Gas-phase diffusion coeff. - 0.1 cm2/sec, Lelieveld 1991

      real*8, parameter :: 
     & avogadrl=6.0221367D20          ! Avogadro N * lit/cm3 (6.020e-20)
      real*8, parameter :: 
     & atmos=2.247D19               ! Atmos. surface pressure mol/cm3
      real*8, parameter ::
     & pii=3.141592653559  !  PI           
      real*8, parameter :: 
     & rtcon=8.314D-02         ! Conversion param for Henry  L-atm/mol-K
      real*8, parameter :: 
     & RUMOLEC=8.314D7             ! RU g-cm2/s2-mol-K
      real*8, parameter ::      ! Droplet diffusion coefficient
     & DROPDIF=2.0D-05             ! 2e-5 cm2/sec, Lelieveld+Crutzen 1991
      real*8, parameter ::      ! Gas-phase diffusion coeff.
     & DIFGAS=0.1D0            ! 0.1 cm2/s Lelieveld and Crutzen, 1991.
c
c
c LOCAL CHEMICAL SOLVER VARIABLES:  
c PRODUCTION AND LOSS RATES AND INTERIM CONCENTRATIONS
c SINGLE SPECIES AND LINKED PAIR SOLUTIONS
c  (OPTION - iteration counter is in common for written output)
c
c   Note:  Allocate.  
c   Moved to ichemlocarr.h  
c cpm, cpro, geomavg, history, rlm, rlmulti, rloss, rloss1,
c  rlpair, rpm, rpmulti, rppair, rpro, rpro1, 
c  rrp, rrl, xrm, xrp, xrppair, xrr, xrrm, xrrpair 
c
c VARIABLES ASSOCIATED WITH ODD HYDROGEN RADICALS AND ODD NITROGEN
c
c   Note - Allocate?  No.
c
c foh(kk)         OH/HO2 fraction. (ohsolv and presolve. not common)
c
c oddhdel(kk,2)     Summed sensitivity of net production of Hx to OH,
c                    summed as PHx* (d ln (pHx)/d ln([OH]))  (1=w RO2 2=w/o RO2)
c                    (molec/cm3)
c                    (This results in a linearized update of Hx:
c                     PHx = oddhsum + oddhdel(OH-OHp))
c oddhloh(kk,2)       Partial sum of net production/loss of Hx, molec/cm3
c                    only from reactions that remove OH
c oddhlho2(kk,2)       Partial sum of net production/loss of Hx, molec/cm3
c                    only from reactions that remove HO2
c oddhsrc(kk,2)       Sumed production of Hx (molec/cm3)
c                                   from net production reactions only
c                  based on prior species concentrations and reactions
c oddhsum(kk,2)       Sum of net production/loss of Hx, molec/cm3
c                  based on prior species concentrations and reactions
c
c oddhsump(kk)      Sum of net production/loss of Hx from prior iteration
c oddhfacp(kk)      Hx concentration adjustment from prior iteration
c
c (oddnsum(kk) - deleted)
c
c senhcat(kk,icat) Estimated ssensitivity of odd-H species conc. 
c                    to changes in OH, by species category.  
c                     d ln [odd-h spec]/d ln([OH])
c                     (Zero for non-odd-H species)
c                     (Sometimes used also for NOx)
c senshx(kk,2)       Sensitivity of Hx production to OH (w/ and w/o RO2)
c             for individual reaction: d ln (pHx)/d ln([OH])
c
c sourcnx(kk)      Summed NOx source from chemistry (molec/cm3/timestep)
c                   (includes net NOx source from PAN reactions)
c sinknx(kk)       Summed NOx sink   from chemistry (molec/cm3/timestep)
c                   (includes net NOx sink   from PAN reactions)
c
c xfohtest         Convergence test for OH/HO2 ratio (nonvectorized)
c                     =((OH/HO2)p-(OH/HO2))/(OH/HO2p
c
c THE FOLLOWING ARE MOVED TO chemvars.EXT: 
c xnotest          Convergence test ratio for NOx (nonvectorized)
c                     =(NOxp-NOx)/NOxp
c xohtest          Convergence test ratio for OH (nonvectorized)
c                     =(OHp-OH)/OHp

      real*8 :: foh(c_kvec)       ! OH/HO2.  not common

c      real*8 ::        oddhdel(c_kvec,2)   ! Summed Hx sens to Oh, mol/cm3
c      real*8 ::        oddhloh(c_kvec,2)   ! Hx loss from OH,  mol/cm3
c      real*8 ::        oddhlho2(c_kvec,2)   ! Hx loss from HO2,  mol/cm3
c     real*8 ::        oddhsrc(c_kvec,2)   ! Summed P Hx, mol/cm3
c      real*8 ::        oddhsum(c_kvec,2)   ! Summed net P/L Hx, mol/cm3
c      real*8 ::        senshx(c_kvec,2)   ! Hx sens: d ln(pHx)/d ln([OH])
c      real*8 ::        senhcat( c_kvec,99) ! Hx sens. by species categ.

c      real*8 :: sourcnx(c_kvec)   ! Summed NOx source, mol/cm3
c      real*8 :: sinknx(c_kvec)    ! Summed NOx sink, mol/cm3

c      real*8 :: xfohtest          ! OH/HO2 convergence test 
c     real*8 :: xnotest           ! NOx convergence test ratio
c     real*8 :: xohtest           ! OH convergence test ratio

c     common/ODDHX/ senshx, senhcat, oddhsum, oddhdel
c    *  , oddhsrc, oddhloh, oddhlho2
c    *  ,sourcnx, sinknx 
c    *  , xfohtest
c c  ,xnotest,xohtest - moved to chemvars
c
c SPECIES INDICES AND COUNTERS FOR CHEMISTRY SOLVER. 
c  (Mostly local,not in common - except 'nsdim' - elim. in future.

c ncsol(nc)         Chem. species number for head of pair group
c                       being solved for.  
c                       NOT IN COMMON.  This is passed to chemsolve.
c 
c ncsolv(nc)        Species number for each species in solver call, 
c                    including subspecies in pair chains,   
c                    nc is number within the solver.
c                    ncsolv(nc) gives the species number (ic)
c                    (Local in chemsolve)
c
c nssolv(nc)        Pair/multisolve group number for each species 
c                    in solver call, including subspec in pair chains.
c                    nc is number within the solver.
c                    nssolv(nc) gives the pair group number (is) 
c                    (Local in chemsolve)
c
c nsol               Number of pair groups to be solved for
c                      (Used in chemread, chemsolve, not in common)
c
c nsolv              Number of species to be solved for 
c                      in individual call to solver (chemsolve)
c                      (Used in chemread, chemsolve, not in common)
c ncdim              Maximum number of total species to be solved
c                       simultaneously in pair or multisolve
c                       (Chemread only. May be unnec.)
c nsdim              Maximum number of species pair groups to be solved
c                       simultaneously in pair or multisolve
c                       (excluding pair chains).
c       (in quadchem, chemsolve, chemread. Must be in common for now.)
c
      integer :: nsol           ! Number of pair groups being solved for
      integer :: nsolv          ! Number of species being solved for
c      integer :: nsdim          ! Maximum number of pair groups being solved
c      integer :: ncdim          ! Maximum Number of species being solved for
      integer :: ncsolv(c_cdim) ! Species number (ic) for species in solver
      integer :: nssolv(c_cdim) ! Pair group number (is) for spec. in solver

c c      common/CHEMX/ncdim, nsdim
c

c LOCAL VARIABLES USED IN CHEMISTRY SUBROUTINES 
c        (not passed between subroutines or included in common block)
c 
c prior (kk)   General vector vbl for prior species concentrations
c                                                    (molec/cm3)
c                 (used for initial concentration or prior estimate)
c alpha (kk)   General vector vbl 
c beta  (kk)   General vector vbl
c gamma (kk)   General vector vbl
c

c stoicx                             stoichiometry sum
c ionsum                              aquasolve ion counter
c ic, ic1, ic2,ic3, icc, ics, ics2, icc1, icc2, icc3, is, iss, iscs
c  ich, icq
c  icx, icx1, icy1, icx2, icy2, icp, icp1, icp2:   
c ica1, ica2, icb1, icb2, nra1,nra2,nrb1,nrb2 
c                                Chem species index numbers
c  neq                          Aqueous equilibrium counters
c nc, nc1, nc2, ncc, ncf, nn, nne           Chem. species counters
c icat1, icat2, icatp, icatp2     indices used for species categories
c nr, nr1, nr2, nrh, nrq, nrqq, nrx, np  :          Reaction counters
c kk       :   Vectorization counter
c kw       :   Index for vector diagnostic output (see c_kkw)

      real*8 :: alpha(c_kvec)   ! General vector variable
      real*8 ::  beta(c_kvec)   ! General vector variable
      real*8 :: gamma(c_kvec)   ! General vector variable
      real*8 :: prior(c_kvec)   ! Prior species conc (molec/cm3)


      real*8 :: stoicx                         ! stoichiometry sum
      integer :: ic,ic1,ic2,ic3,icc,ics,ics2,icc1,icc2,icc3 !Chem index
      integer :: is,iss,iscs,isc2                        !Chem local index
      integer :: ich,icq,icx,icx1,icx2,icy1,icy2,icp,icp1,icp2 !Chem index
      integer :: icr1,icr2,icr,isr1,isr2,icpair  !chem index - pair and multi
      integer :: ionsum                                  ! aquasolve ion counter
      integer :: ica1,ica2,icb1,icb2,nra1,nra2,nrb1,nrb2 !Aquasolve chem index
      integer :: neq                                     !Aqueous counters
      integer :: nc,nc1,nc2,ncc,ncf,nn,nne               !Chem species counters
      integer :: nr,nr1,nr2,nrh,nrq,nrqq,nrx,np          ! Reaction counters
      integer :: icat1,icat2,icatp,icatp2  ! indices used for species categories
      integer :: kk, kw                            ! Vectorization counters
      integer :: i, j, k, ii, ij, iii, n           ! General counters
c 
c
c CUT LOCAL VARIABLES  - Local in individual subroutines
c
c   lloss        Local: Flag for identifying exchange loss reaction
c                         (cheminit)
c   lpro         Local: Flag for identifying exchange production  reaction
c                         (cheminit)
c   hvrate(kk, 56)    Interpolated hv rates averaged over time interval.
c                                      (sec-1)  (bhvrates)
c   fhvint            Fraction assigned to each discrete time period
c                       used to interpolate for the full interval
c                       (=1/ihvint) (bhvrates)
c   ihvint            Number of discrete times  used  to interpolate
c                       hv over the simulated time interval  (bhvrates)
c 
c   jval(      56)    J-value output from jval2.f parameterization.
c                                      (sec-1)  (bhvrates)
c   zen(kk)           Zenith angle (degrees) (90=horizon) (bhvrates)
c
c   ncsol(c_cdim)     Species list passed in call to chemsolve

c
c ----------------------------------------
c END OF chemlocal.EXT
c ----------------------------------------
c ================================================================
         end module h_rbchemlocal


