       module h_rbchemmech
       implicit none
       public

c=============================================================================
c
c $Id: rbchemmech.h,v 1.4 2002/05/02 15:50:22 akinorii Exp $
c
c CODE DEVELOPER
c   Sandy Sillman, U of M. (IMPACT version, 2007)
c
c FILE
c   rbchemmech.h
c
c DESCRIPTION:  variable declarations and common blocks
c   for variables and arrays (non-allocated) relating to the chem. mechanism
c   used in the rb (quad) solver.
c   Includes rate parameters, TUV table for j-values, 
c    and derived chem. structure for use in rb solver.
c
c   These variables are all input or calculated once, at the start, 
c    and then used whenever the chem. solver is used.
c   They  must be saved and made available when the solver is invoked.
c
c NOTE FOR IMPACT MESSAGE PASSING: 
c   From Akinori and Dan Bergman:  
c   Common blocks can be used only for values that are identical 
c    for all grids.  
c   Common blocks here are all for chemistry parameters and remain constant
c    after the initial calculation or input read.
c   Common blocks here are separated based on variable type (integer, etc.)
c
c ---
c rbchemmech.h 09
c  This has added indices for SPECIAL REACTIONS (NRSPECIAL)
c    for preliminary calculation.c
c
c Also:  
c  c_nco3    Index to identify CO3- in species list
c  c_noxchem(13,nr) = stoich parameters for tracers 
c  c_nrro2  Index for setting rates for combined RO2-RO2 reactions (CBMZ)
c -----
c ==========================================================================
c
c ichemmech.h    from chemmech.EXT    April, 2007    
c   INCLUDE and COMMON file 
c     for RADICAL BALANCE-BACK EULER solver for chemistry (quadchem)
c      (chemmain.f cheminit.f chemrates chemsolve.f, linslv.f, jval2.f)
c
c IMPACT notes:  
C   The first two groups of variables 
c     (READ/WRITE and VECTOR INDICES, and NUMERICAL PARAMETERS)
c     may be changed to be entered here as PARAMETERS 
C     rather than as COMMON variables.  
c     Currently they are included in the INPUT file or set in the call 
c      to the solver.
c
c  This file includes declarations and common blocks
c   for MECHANISM and NUMERICAL VARIABLES:  reaction and j-value data
c   which are read and/or set up  once at the start of the program
c   and then remain unchanged during calls to the solver.
c
c ----------------------------
c  CRITICAL PARAMETERS AND CHOICES:
c   c_kvec, parameter for VECTORIZATION.
c   c_cdim and c_rdim = maximum number of species and reactions
c                       (Currently 1511,3211. Can be increased.)
c
c   Real vs real*8:  (Double precision is standard)
c   If real*8, change 'real' to 'real*8'
c    throughout this file and all fortran files.
c    also change exponents (DO vs EO) in BOD and YTN
c ----------------------------
c
c  Other INCLUDE files: 
c   chemvars.EXT =  input-output variables 
c    that are passed to and from the solver each time it is called.
c   chemlocal.EXT = Variables that are shared among the subroutines
c     in the chemistry solver, but not used elsewhere and not saved.
c
c Program written by Sanford Sillman
c History:
c  12/06. Program written by Sandy Sillman based on commq7.EXT
c  6/07 critical parameters moved from chemvars to chemmech;  
c   IMPACT version as ichemmech.h.
c -------------------------------------------------------------

c -----------------------
c STAND-ALONE PARAMETERS (without common block)
c -----------------------
c ESSENTIAL index parameters:
c                    (values set here; no need for common block)
c     c_kvec:  maximum dimension for vectorization
c     c_cdim: maximum dimension for the number of species
c     c_rdim: maximum dimension for the number of reactions

       integer, parameter ::
     &   c_kvec=1                 ! Maximum dimension for vectorization
c      PARAMETER (c_kvec=31)

       integer, parameter ::  
c     &  c_cdim=211
c----------------gxlin----10-15-2009---------
     &  c_cdim=500
c----------------gxlin----10-15-2009---------
c     &  c_cdim=1511            ! max dimension for number of species
       integer, parameter ::  
c     &  c_rdim=548
c     &  c_rdim=3211   ! max dimension for number of reactions
c------------gxlin-------10-15-2009----------
c     &  c_rdim=611
     &  c_rdim=1000
c----------------gxlin-------10-15-2009-----
       integer, parameter ::
c     &  c_hdim=   4   ! max dimension for number of Henry's law constants
     &  c_hdim= 100 
       integer, parameter ::
     &  c_aqdim= 100 
c     &  c_aqdim=  4   ! max dimension for number of aqueous equilibria
       integer, parameter ::
     &  c_aqqdim= 100 
c     &  c_aqqdim= 4   ! max dimension for number of special equilibria

c
c ----------------------
c COMMON BLOCK VARIABLES
c ----------------------
c
c READ/WRITE AND VECTOR INDICES
c c_kkw               Index for diagnostic write output (0 for none)
c                     (May be set to write for specific grid and time)
c c_kmax              Maximum number for vectorization
c c_out               Output file index
c c_rin               Input file index for REACTION.DAT (mechanism)
c c_hvin              Input file index for TUVGRID2 (hv data)
c
      integer :: c_kkw  ! Index for diagnostic write output (0 for none)
     * ,      c_kmax    ! Maximum number for vectorization
     * ,      c_out     ! Output file index
     * ,      c_rin     ! Input file index for REACTION.DAT (mechanism)
     * ,      c_hvin    ! Input file index for TUVGRID2 (hv data)
c
c       common/CHEMIO/ c_kkw, c_kmax, c_out ,c_rin, c_hvin

c NUMERICAL SOLUTION  PARAMETERS
c  c_numitr              Maximum number of iterations
c  c_converge            Convergence criteria (1e-02)

      integer :: c_numitr               ! Maximum number of iterations
      real*8 :: c_converge    ! Convergence criteria (1E-02)

c      common/CHEMNUMERI/ c_numitr
c      common/CHEMNUMER/ c_converge
c
c REACTION RATE PARAMETERS:  
c
c  c_rk      Gas-phase rate parameters (for calculating rate:  up to 7)
c            connected with rate calc. options in chemrates
c  c_rkh     Henry's law rate parameters 
c  c_rkq     Aqueous equilibrium parameters (A=B+C, H+ and OH- reactions)
c  c_rkqq    Special equilibrium parameters
c 
c  c_nrk(c_rdim)    Format index for reaction rate
c                    This ID's read format and reaction rate formula
c  c_nrkh(c_rdim)   Format index for Henry's law coefficient
c  c_nrkq(c_rdim)   Format index for regular equilibrium constant
c  c_nrkqq(c_rdim)  Format index for special equilibrium constant
c
c  c_accom(c_rdim)   Accomodation coefficients
c  c_molwt(c_rdim)   Molecular weights


      real*8           ::  
     &                 c_rk(7,c_rdim)    ! gas-phase rate parameters
     & ,               c_rkh(2,c_hdim)   ! Henry's law parameters
     & ,               c_rkq(2,c_aqdim)   ! Aqueous equilib. params.
     & ,               c_rkqq(2,c_aqqdim)  ! Special equilib. params.
     & ,               c_accom(c_cdim)   ! Accomodation coefficients
     & ,               c_molwt(c_cdim)   ! Molecular weights
c
      integer ::  
     &        c_nrk(c_rdim)    ! Format index for reaction rate
     & ,      c_nrkh(c_hdim)   ! Format index for Henry's law coef.
     & ,      c_nrkq(c_aqdim)   ! Format index for equilibrium constant
     & ,      c_nrkqq(c_aqqdim)  ! Format index for special eq constant
c

c       common/CHEMRVARS/ c_rk, c_rkh, c_rkq, c_rkqq, 
croy     *                   c_nrk, c_nrkh, c_nrkq, c_nrkqq,
c     *                   c_accom, c_molwt
c       common/CHEMRVARSI/ c_nrk, c_nrkh, c_nrkq, c_nrkqq
c
c CHEMICAL MECHANISM AND SPECIES INDICES
c  c_reactant(c_rdim,2)   Index for reactant species in A+B=C..
c  c_product(c_rdim,20)   Index for product species in A+B=C...
c  c_nnpro(c_rdim)        number of reaction products for reaction nr. 
c  c_stoich(c_rdim,20)    Stoichiometry for each product
c  c_prodarr(nr,ic)       Array of stoichiometry for production of chem ic 
c                         from reaction nr - assigned to gas species.
c                         (used in solution for chem pairs and multisolve
c
c  c_henry(c_rdim,2)      Index for Henry's law species: A<=>B
c  c_aqueous(c_rdim,3)    Index for aqueous equil. species: A<=>B+C
c  c_aqspec(c_rdim,3)     Index for special equil. species: A<=>B+C
c
c  c_nreac     Number of chemical reactions
c  c_nreach    Number of Henry's law equilibrium constants
c  c_nreacq    Number of regular equilibrium constants (w/H+, OH-)
c  c_nreaqq    Number of special equilibrium constants
c
c  c_nchem1    Number if input/output (transported) species
c  c_nchem2    Number of total species 
c 
c  c_icat           Category of chemical species
c  c_lsts           Flag to identify steady state species (T for stst)
c  c_lump(nlump,3)  Lumped species: species n1= sum from n2 to n3
c  c_llump    Flag to identify lumped species accumulator (n1) (T for n1)
c
c  (nk() and iarray() - from old NR solver - cut.)

      integer ::
     &        c_reactant(c_rdim,2)      ! Reactant species numbers
     & ,      c_product(c_rdim,20)      ! Product  species numbers
     & ,      c_nnpro(c_rdim)           ! Number of reaction products
      real*8 ::
     &        c_stoich(c_rdim,20)       ! Stoich. for products
     & ,      c_prodarr(c_rdim, c_cdim) ! Stoich. nr for ic
      integer ::  
     &        c_henry(c_hdim,2)         ! Henry's law species numbers
     & ,      c_aqueous(c_aqdim,3)       ! Aqueous equil. species numbers
     & ,      c_aqspec(c_aqqdim,3)        ! Special equil. species numbers

      integer ::
     &        c_nreac    ! Number of chemical reactions
     & ,      c_nreach   ! Number of Henry's law equilibrium constants
     & ,      c_nreacq   ! Number of regular equilibrium constants 
     & ,      c_nreaqq   ! Number of special equilibrium constants
c
      integer :: 
     &        c_nchem1   ! Number if input/output (transported) species
     & ,      c_nchem2   ! Number of total species 
     & ,      c_icat(c_cdim)  ! Category of chemical species
     & ,      c_lump(c_cdim,3)  ! Lumped species:  n1= sum n2 to n3
      logical c_llump(c_cdim)  ! Flag to identify lumped species (n1)
      logical c_lsts(c_cdim)  ! Flag to identify steady state species

c      common /CHEMMECH/ c_reactant, c_product, c_nnpro
c     * ,  c_aqueous,  c_henry, c_aqspec 
croy     * ,  c_stoich, c_prodarr
c     * ,  c_nreac, c_nreach,c_nreacq,c_nreaqq
c     * ,  c_nchem1, c_nchem2, c_icat, c_lsts, c_lump
c     & ,  c_llump
c      common /CHEMMECH2/ c_stoich, c_prodarr
c 
c INDICES FOR CHEMICAL REACTION SOLUTION PROCEDURE (CASCADE)

c  c_cascade(ic,n)  Array of cascade species: identified species 
c                   in order of solution and single/multisolve
c
c  c_nppair(c_cdim,23)  pair pointers: for linked pair chain in cascade
c  c_nppair(ic,1)       pointer to directly linked pair species
c                        (to species directly above in pair hierarchy)
c  c_nppair(ic,2)       pointer to main primary species for pair group
c  c_nppair(ic,3)       number of pair group subspecies for this species
c  c_nppair(ic,4+)      chem. index for pair group subspecies of species ic 
c  c_npmulti(ic,1)      chem index for head of multi group for this spec.
c
c  c_nnrchem(c_cdim)        Cascade solution order: 
c                             number of reactions linked to species ic
c  c_nrchem(c_cdim,c_rdim)  Cascade solution order:
c                             Reaction numbers (nr) linked to species ic
c  c_nnrchp(c_cdim)         Cascade solution order: 
c                             number of reactions linked to ic as  product
c  c_nrchmp(c_cdim,c_rdim)  Cascade solution order:
c                             Reaction numbers (nr) linked to ic as product
c  n_nicreac(c_rdim)        Chemical species associated with reaction.
c
c  c_exloss(ic, is,i)  Reaction number for 'exchange' loss reaction: 
c                        loss to species ic;  counter for number of 
c                        exchange species is, number of reactions i
c  c_expro(ic,is,i)    Reaction number for 'exchange' product reaction:
c                        produces species ic;  counter for number of 
c                        exchange species is, number of reactions i
c  c_exspec(ic,is)     Exchange species number for species #is
c                        that is in back-forth reaction with species ic

      integer ::
     &        c_cascade(c_cdim,c_cdim), ! Array to call cascade
     &        c_nppair(c_cdim,23),      ! Pointers to paired species
     &        c_npmulti(c_cdim,1),      ! Pointer to multi spec. head
     &        c_nnrchem(c_cdim),        ! Number of reactions linked to spec.
     &        c_nrchem(c_cdim, c_rdim), ! Reaction numbers for spec. 
     &        c_nnrchp(c_cdim),         ! Number of product reactions for ic
     &        c_nrchmp(c_cdim, c_rdim), ! Productreaction numbers for ic
     &        c_nicreac(c_rdim),        ! Species ic associated with reaction nr
     &        c_exloss(c_cdim,20,5),    ! Exchange loss reaction number
     &        c_expro(c_cdim,20,5),     ! Exchange product reaction number
     &        c_exspec(c_cdim,20)       ! Exchange species number

c     common/CHEMSOL/ 
c      common/CHEMCASCADE/ 
c     *                  c_cascade, c_nppair, c_npmulti
c     *                 , c_nnrchem, c_nrchem, c_nnrchp, c_nrchmp
c     *                 , c_nicreac
c     *                 , c_exloss, c_expro,  c_exspec
c ADDED INDICES FOR SOLUTION PROCEDURE (from rbchemlocal.h)
c ncdim                 Maximum number of total species
c               to be solved simultaneously in pair or multisolve
c                        (Chemread only. May be unnec.)
c nsdim                 Maximum number of species pair groups
c               to be solved simultaneously in pair or multisolve
c                   (excluding pair chains).
c       (in quadchem, chemsolve, chemread. Must be in common for now.)

      integer :: ncdim         ! Maximum Number of species being solved for
      integer :: nsdim         ! Maximum number of pair groups being solved

c      common/CHEMX/ncdim, nsdim

c
c  CHEMISTRY REACTION COUNTERS:  
c   c_stoiloss(c_rdim)  counter for net loss of reactant per reaction
c                         (used to ID self-reaction or product=reactant)
c   stoicpro, stoicprx - cut, no longer used
c
c   c_oddhx(c_rdim,n):    counter for net prod/loss of odd-H per reaction
c                        n=1 with RO2 n=2 w/o RO2
c   c_pronox(c_rdim)    Counter for net production of NOx per reaction.
c                         (net production only; not loss)
c c_pairfac(c_cdim)     Pair group conservation  of mass factor per spec.
c                         Counts double for C=A+B, for A, B in pair group
c c_multfac(c_cdim)     Multi group conservation  of mass factor per spec.
c                         Counts double for C=A+B, for A, B in pair group
c
      real*8 ::
     &                 c_stoiloss(c_rdim)  ! net reactant loss
     & ,               c_oddhx(c_rdim,2)     ! counter for net odd-H
     & ,               c_pronox(c_rdim)    ! counter for net P(NOx)
     & ,               c_pairfac(c_cdim)  ! Pair group net rp factor
     & ,               c_multfac(c_cdim)  ! Multi group net rp factor

c      common/CHEMCOUNT/ c_stoiloss, c_oddhx,  c_pronox,
c     *                  c_pairfac, c_multfac
c
c INDICES AND POINTERS FOR AQUEOUS EQUILIBRIA
c  c_nequil(c_cdim)     Number of aqueous/eq species linked to gas ic
c  c_npequil(c_cdim)    Pointer to linked gas-master species
c  c_ncequil(c_cdim,6)  List of aqueous/eq species for each gas-master
c  c_nrequil(c_cdim,6)  Henry's law number (nrh) for first aqueous spec,
c                         Aqueous equil. number (nrq) for subsequent spec,
c                         for species associated with each gas-master.
c  c_ion(c_cdim)        Charge associated with each species
c 
      integer ::
     &        c_nequil(c_cdim)    ! Number of aq species linked to gas ic
     & ,      c_npequil(c_cdim)   ! Pointer to linked gas-master species
     & ,      c_ncequil(c_cdim,6) ! List of aq species for gas-master
     & ,      c_nrequil(c_cdim,6) ! nrh/nrq for linked aqueous spec.
     & ,      c_ion(c_cdim)       ! Charge associated with each species
c
c       common/CHEMEQUILX/ c_nequil, c_ncequil, c_nrequil, c_npequil, 
c     *                     c_ion
c
c INDICES AND POINTERS FOR AQUEOUS EQUILIBRIA
c 
c CHEM MECHANISM NAMES  tchem
c  c_tchem(c_cdim)          Chem. species name, a8
c  c_treac(6, c_rdim)       Chem reaction species names: A+B->3 products
c  c_treach(2, c_hdim)      Henrys law species names: A<=>B
c  c_treacq(3, c_aqdim)      Equilibrium spec names:  A<=>B+C
c  c_treacqq(3, c_aqqdim)     Special eq spec names:   A<=>B+C

      character*8 ::
     &            c_tchem(c_cdim)         ! Chem. species name
     & ,          c_treac(6, c_rdim)      ! Chem reaction species names
     & ,          c_treach(2, c_hdim)     ! Henrys law species names
     & ,          c_treacq(3, c_aqdim)     ! Equilibrium spec names
     & ,          c_treaqq(3, c_aqqdim)     ! Special eq spec names

c       common/CHEMNAMES/ c_tchem, c_treac, c_treach, c_treacq, c_treaqq
c
c CHEM INDICES FOR SPECIAL SPECIES: To identify H2O, H+, OH, etc.

c  c_nh2o    Index to identify H2O  in species list
c  c_nco2    Index to identify CO2  in species list
c  c_nhplus  Index to identify H+   in species list
c  c_nohmin  Index to identify OH-  in species list
c  c_noh     Index to identify OH   in species list
c  c_nho2    Index to identify HO2  in species list
c  c_nh2o2   Index to identify H2O2 in species list
c  c_no3     Index to identify O3   in species list
c  c_nno2    Index to identify NO2  in species list
c  c_nno     Index to identify NO   in species list
c  c_nno3    Index to identify NO3  in species list
c  c_nn2o5   Index to identify N2O5 in species list
c  c_nhno3   Index to identify HNO3 in species list
c  c_nh2     Index to identify H2   in species list
c  c_nco     Index to identify CO   in species list
c  c_nch4    Index to identify CH4  in species list
c  c_nco3    Index to identify CO3- in species list
c  c_no2     Index to identify  O2  in species list
c 
      integer ::
     &        c_nh2o   ! Index to identify H2O  in species list
     & ,      c_nco2   ! Index to identify CO2  in species list
     & ,      c_nhplus ! Index to identify H+   in species list
     & ,      c_nohmin ! Index to identify OH-  in species list
     & ,      c_noh    ! Index to identify OH   in species list
     & ,      c_nho2   ! Index to identify HO2  in species list
     & ,      c_nh2o2  ! Index to identify H2O2 in species list
     & ,      c_no3    ! Index to identify O3   in species list
     & ,      c_nno2   ! Index to identify NO2  in species list
     & ,      c_nno    ! Index to identify NO   in species list
     & ,      c_nno3   ! Index to identify NO3  in species list
     & ,      c_nn2o5  ! Index to identify N2O5 in species list
     & ,      c_nhno3  ! Index to identify HNO3 in species list
     & ,      c_nh2    ! Index to identify H2   in species list
     & ,      c_nco    ! Index to identify CO   in species list
     & ,      c_nch4   ! Index to identify CH4  in species list
     & ,      c_nco3   ! Index to identify CO3- in species list
     & ,      c_no2    ! Index to identify O2    in species list

c      common/CHEMSPECIAL/ c_nh2o, c_nco2, c_nhplus, c_nohmin
c     *       ,c_noh, c_nho2, c_nh2o2, c_no3, c_nno2, c_nno
c     *       ,c_nno3, c_nn2o5, c_nhno3
c     *       ,c_nh2,   c_nco,  c_nch4, c_nco3,c_no2


c NOVEMBER 2007 ADDITION:
c   chemistry mechanism parameters to be used in tracer calculations
c
c STOICHIOMETRY PARAMETERS used in global tracer calculations
c  c_noxchem(13,nr) = stoich parameters for:
c                1-5: net rp Ox; NOx, PAN, HNO3, RNO3;
c                6-13:   NOx->PAN, PAN->NOx;  N<->HNO3;  N<->RNO3, N<=>H+RNO3
c
c       common/RPSTOIC/ c_noxchem        ! declared just below

c REACTION NUMBERS IDENTIFIED FOR SPECIAL REACTIONS.
c   (used in preliminary calculation, oh solution, and global tracers)
c   (2009 addition: include nr for RO2-RO2 reactions for CBMZ special rate cal

c  c_nro3no   = nr for O3+NO=>NO2
c  c_nrno2x   = nr for NO2+hv=>NO+O3

       real*8 ::        c_noxchem(13, c_rdim)   ! net rp Ox NOx PAN HNO3 RNO3
       integer :: c_nro3no                         ! nr for O3+NO=>NO2
       integer :: c_nrno2x                         ! nr for NO2+hv
       integer :: c_nro3hv                         ! nr for  O3+hv->2OH
       integer :: c_nrohno2                        ! nr for OH+NO2->HNO3
       integer :: c_nrohco                         ! nr for OH+CO
       integer :: c_nrho2no                        ! nr for HO2+NO
       integer :: c_nrho22                         ! nr for HO2+HO2
       integer :: c_nro3no2                        ! nr for O3+NO2->NO3
       integer :: c_nroho3                         ! nr for OH+O3
       integer :: c_nrohch4                        ! nr for OH+CH4
c 2009 addition
       integer :: c_nnrro2                ! Number of special RO2-RO2 reactions

       integer :: c_nrro2(c_rdim)         ! nr for special RO2-RO2 reactions

c       common/NRSPECIAL/ c_nro3no, c_nrno2x
c     &  , c_nro3hv, c_nrohno2, c_nrohco, c_nrho2no
c     &  , c_nrho22, c_nro3no2, c_nroho3, c_nrohch4
c     &  , c_nnrro2, c_nrro2


c HVRATES:  Data input for J-value parameterization
c            (jval2.f, developed at Michigan based on Madronich TUV)
c
c Note: jval(56)       Output from J-value parameterization.
c                        Declared internally in hvrates
c  c_nhv(ipar)         Number of parameter intervals for each parameter
c  c_hvmat(ipar,ialt)  Value of parameter interval points 
c                        for each of 22 parameters, 40 altitudes
c  c_hvmatb(ipar)      Values for temperature adjustment
c  c_jarray( k,ig,jc)  Output array for 56 jvalues (jc)
c                       for matrix of 510 altitudes and zenith angles (ig)
c                       and 80 conditions (k): 1=base value, >1 adjustments

       integer ::  c_nhv(22)                   
       real*8 ::
     &                  c_hvmat(22,40)     
     & ,                c_hvmatb(22)        
     & ,                c_jarray(80,510,56)

c       common/HVRVARS/       c_hvmat, c_hvmatb, c_jarray
c       common/HVRVARI/c_nhv
c 
c ----------------------------------------
c END OF chemmech.EXT
c ----------------------------------------
c ================================================================

          end module h_rbchemmech
